package com.abi.task.common.excel;

import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.abi.task.common.excel.common.AbstractExcelProperty;
import com.abi.task.common.excel.common.ExcelField;
import com.abi.task.common.excel.common.ExcelHeadAlias;
import com.abi.task.common.utils.PojoConverterUtils;
import com.abi.task.common.utils.StrUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.util.CollectionUtils;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * excel导出工厂类
 * @Author AndyTan
 * @Date 2021/1/18
 */
@Slf4j
public class ExcelOutputFactory {

    private ExcelOutputFactory() {}

    /**
     * 获取某个excel模板的所有字段
     * @param clz
     * @param <T>
     * @return
     */
    public static <T extends AbstractExcelProperty> List<ExcelField> getExcelColumns(Class<T> clz) {
        List<ExcelField> result = new LinkedList<>();

        Field[] fields = clz.getDeclaredFields();
        for(Field field:fields){
            String fieldName = field.getName();
            ExcelHeadAlias headAlias = field.getAnnotation(ExcelHeadAlias.class);
            String fieldDesc = headAlias.value();

            result.add(new ExcelField(fieldName,fieldDesc));
        }

        return result;
    }

    /**
     * 重载方法导出所有字段
     * @param excelFileName
     * @param response
     * @param businessList
     * @param clz
     * @param <T>
     */
    public static <T extends AbstractExcelProperty> void outputStreamExcel(
            String excelFileName,
            HttpServletResponse response,
            List businessList,
            Class<T> clz){
        List<String> excelColumns = getExcelColumns(clz).stream().map(excelField -> excelField.getFieldName()).collect(Collectors.toList());
        outputStreamExcel(excelFileName,response,businessList,excelColumns,clz);
    }


    /**
     * 导出前台选择的部分字段
     * @param excelFileName 文件名
     * @param response      servlet的response
     * @param businessList  业务的列表
     * @param excelColumns  前台选择的需要导出的字段
     * @param clz           模板类
     * @param <T>
     */
    public static <T extends AbstractExcelProperty> void outputStreamExcel(
            String excelFileName,
            HttpServletResponse response,
            List businessList,
            List<String> excelColumns,
            Class<T> clz){

        //没有数据就放一个空（一般用于导出模板）
        if(CollectionUtils.isEmpty(businessList)){
            try {
                businessList = new ArrayList();
                businessList.add(clz.newInstance());
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }

        //1-创建excel
        ExcelWriter writer = ExcelUtil.getWriter(true);
        //放入标题头
        addHeaderAliasMethod(writer,clz);
        //转化为map
        List<Map> excelMapList = toMapList(businessList,excelColumns,clz);
        //字段排序比较器
        Comparator<String> columnComparator = getColumnComparator(clz);
        //创建excel
        writer.write(excelMapList, columnComparator);
        //用描述宽度去计算宽度
        Field[] fields = clz.getDeclaredFields();
        for(int i=0;i<fields.length;i++){
            Field field = fields[i];
            //字段描述
            ExcelHeadAlias headAlias = field.getAnnotation(ExcelHeadAlias.class);
            String fieldDesc = headAlias.value();
            writer.setColumnWidth(i,fieldDesc.length()*2+5);
        }

        //2-输出流
        //COR
        response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_CREDENTIALS, "true");
        response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_ORIGIN, "*");
        response.setHeader(HttpHeaders.ACCESS_CONTROL_EXPOSE_HEADERS, "Cache-Control,Content-Language,Content-Type,Expires,Last-Modified,Pragma,Content-Disposition");
        //指定字符集
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        //指定下载名字拼接，主要是前台传进来的数据
        ServletOutputStream out = null;
        try {

            String fileName = excelFileName+".xlsx";
            response.setHeader("Content-Disposition", "attachment;filename="+ URLEncoder.encode(fileName,"utf-8"));
            out = response.getOutputStream();
            writer.flush(out, true);
        } catch (IOException e) {
            log.error("IOException={}", e);
        } finally {
            // 关闭writer
            writer.close();
            // 关闭输出Servlet流
            IoUtil.close(out);
        }
    }

    /**
     * 增加标题行
     * @param writer
     * @param clz
     */
    private static void addHeaderAliasMethod(ExcelWriter writer,Class clz) {
        Field[] fields = clz.getDeclaredFields();
        for(Field field:fields){
            //字段名
            String fieldName = field.getName();
            //字段描述
            ExcelHeadAlias headAlias = field.getAnnotation(ExcelHeadAlias.class);
            String fieldDesc = headAlias.value();
            writer.addHeaderAlias(fieldName, fieldDesc);
        }
    }

    /**
     * 业务对象转excelPreperty对象
     * @param businessList
     * @param clz
     * @param <T>
     * @return
     */
    private static <T extends AbstractExcelProperty> List<T> toExcelPropertyList(List businessList,Class<T> clz){
        List<T> excelPreopertyList = new LinkedList<>();
        for(Object obj:businessList){
            T t = PojoConverterUtils.copy(obj,clz);
            excelPreopertyList.add(t);
        }

        return excelPreopertyList;
    }

    /**
     * businessList转为mapList
     * @param businessList
     * @param t
     * @param <T>
     * @return
     */
    private static <T extends AbstractExcelProperty> List<Map> toMapList(List businessList,List<String> excelColumns,Class<T> t){
        List<Map> mapList = new LinkedList<>();

        //前台希望导出的字段
        HashSet<String> excelColumnSet = new HashSet<>();
        if(!CollectionUtils.isEmpty(excelColumnSet)){
            excelColumnSet.addAll(excelColumns);
        }
        //循环放入map
        for(Object obj:businessList){
            Map map = new HashMap(16);
            //字段数组
            Field[] feilds = t.getDeclaredFields();
            putFieldName(excelColumnSet, map, feilds, obj);
            mapList.add(map);
        }

        return mapList;
    }
    private static Map putFieldName(HashSet<String> excelColumnSet, Map map, Field[] feilds, Object obj) {
        for(Field field:feilds){
            try {
                String fieldName = field.getName();
                //如果前台不想导出这个字段，就跳过(如果前台给了个空列表，就导出全部)
                if(!CollectionUtils.isEmpty(excelColumnSet) && !excelColumnSet.contains(fieldName)){
                    continue;
                }
                //方法map
                Map<String,Method> methodCache = new HashMap<>(16);
                //获取method
                if (!methodCache.containsKey(fieldName)) {
                    methodCache.put(fieldName, obj.getClass().getMethod("get" + fieldName.substring(0, 1).toUpperCase() + fieldName.substring(1)));
                }
                Method method = methodCache.get(fieldName);
                if (method == null){
                    continue;
                }
                //反射调用拿到结果
                Object value = null;
                if (obj instanceof String){
                    value = method.invoke(StrUtils.filterHtml(obj.toString().replaceAll(" ","")));
                }else{
                    value = method.invoke(obj);
                }
                map.put(fieldName, value);
            }catch (Exception e){
                log.info("toMapList error",e);
            }
        }
        return map;
    }

    /**
     * 字段排序比较器（解决map为入参没法排序问题）
     * @param clz
     * @param <T>
     * @return
     */
    private static <T extends AbstractExcelProperty> Comparator<String> getColumnComparator(Class<T> clz){
        Field[] fields = clz.getDeclaredFields();
        Map<String,Integer> columnIndexMap = new HashMap<>(16);
        for(int i=0;i<fields.length;i++){
            columnIndexMap.put(fields[i].getName(),i);
        }

        return new Comparator<String>() {
            @Override
            public int compare(String o1, String o2) {
                Integer o1Index = columnIndexMap.get(o1);
                Integer o2Index = columnIndexMap.get(o2);
                return o1Index.compareTo(o2Index);
            }

            @Override
            public boolean equals(Object obj) {
                if(obj == null){
                    return false;
                }
                return obj.equals(this);
            }

            @Override
            public int hashCode() {
                return super.hashCode();
            }
        };
    }



}
