package com.abi.task.common.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * 对象转换工具类
 * @author Andy.Tan
 */
@Slf4j
public class PojoConverterUtils {

    private PojoConverterUtils() {}

    /**
     * 将源POJO对象数据复制给目标POJO对象的同名属性
     *
     * @date 2018年12月4日
     * @version 1.0
     *
     * @param source 源
     * @param targetClass 目标
     * @return 目标
     */
    public static <S, T> T copy(S source, Class<T> targetClass) {
        try {
            T instance = targetClass.getDeclaredConstructor().newInstance();
            BeanUtil.copyProperties(source, instance);
            return instance;
        } catch (ReflectiveOperationException e) {
            e.printStackTrace();
            throw new RuntimeException("cannot instantiation class");
        }
    }

    /**
     * 将源POJO对象数据复制给目标POJO对象的同名属性
     *
     * @date 2018年12月4日
     * @version 1.0
     *
     * @param source 源
     * @param targetClass 目标
     * @return 目标
     */
    public static <S, T> T copy(S source, Class<T> targetClass, String... ignoreProperties) {
        try {
            T instance = targetClass.getDeclaredConstructor().newInstance();
            BeanUtil.copyProperties(source, instance, ignoreProperties);
            return instance;
        } catch (ReflectiveOperationException e) {
            e.printStackTrace();
            throw new RuntimeException("cannot instantiation class");
        }
    }

    /**
     * 将源POJO对象数据复制给目标POJO对象的同名属性
     *
     * @date 2018年12月4日
     * @version 1.0
     *
     * @param source 源
     * @param target 目标
     * @return 目标
     */
    public static <S, T> T copy(S source, T target) {
        BeanUtil.copyProperties(source, target);
        return target;
    }

    /**
     * 将源POJO对象数据复制给目标POJO对象的同名属性
     *
     * @date 2019-1-29
     * @param source 源
     * @param target 目标
     * @param ignoreProperties 无需转换的属性
     * @param <S>
     * @param <T>
     * @return 目标
     */
    public static <S, T> T copy(S source, T target, String... ignoreProperties) {
        BeanUtil.copyProperties(source, target, ignoreProperties);
        return target;
    }


    /**
     * 将源POJO对象数据复制给目标POJO对象的同名属性
     *
     * @date 2019-1-29
     * @param source 源
     * @param target 目标
     * @param ignoreNullProperties 是否无视null值字段（如果为true则无视）
     * @param <S>
     * @param <T>
     * @return 目标
     */
    public static <S, T> T copy(S source, T target, boolean ignoreNullProperties) {
        BeanUtil.copyProperties(source, target, CopyOptions.create().setIgnoreNullValue(ignoreNullProperties));
        return target;
    }

    /**
     * 对象List复制
     *
     * @param ss
     * @param cls
     * @param <S>
     * @param <T>
     * @return
     */
    public static <S, T> List<T> copyList(List<S> ss, Class<T> cls) {

        List<T> tRes = new ArrayList<>();
        try {
            for (S s : ss) {
                T t = cls.getDeclaredConstructor().newInstance();
                BeanUtil.copyProperties(s, t);
                tRes.add(t);
            }
        } catch (Exception e) {
            log.info("类型转换异常，异常信息: ", e);
        }

        return tRes;
    }


    /**
     * 对象List复制
     *
     * @param ss
     * @param cls
     * @param ignoreProperties
     * @param <S>
     * @param <T>
     * @return
     */
    public static <S, T> List<T> copyList(List<S> ss, Class<T> cls, String... ignoreProperties) {

        List<T> tRes = new ArrayList<>();
        try {
            for (S s : ss) {
                T t = cls.getDeclaredConstructor().newInstance();
                BeanUtil.copyProperties(s, t, ignoreProperties);
                tRes.add(t);
            }
        } catch (Exception e) {
            log.info("类型转换异常，异常信息: Exception={}", e);
        }

        return tRes;
    }

    /**
     * 对象List复制
     *
     * @param ss
     * @param cls
     * @param ignoreNullProperties
     * @param <S>
     * @param <T>
     * @return
     */
    public static <S, T> List<T> copyList(List<S> ss, Class<T> cls, boolean ignoreNullProperties) {

        List<T> tRes = new ArrayList<>();
        try {
            for (S s : ss) {
                T t = cls.getDeclaredConstructor().newInstance();
                BeanUtil.copyProperties(s, t, CopyOptions.create().setIgnoreNullValue(ignoreNullProperties));
                tRes.add(t);
            }
        } catch (Exception e) {
            log.info("类型转换异常，异常信息: ", e);
        }
 
        return tRes;
    }
}