package com.abi.qms.platform.infrastructure.interceptor;

import com.abi.base.foundation.util.RedisClient;
import com.abi.qms.platform.annotation.PassToken;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.infrastructure.util.TokenUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.task.common.api.exception.BusinessException;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.auth0.jwt.JWT;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.Claim;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import static com.abi.qms.platform.infrastructure.constant.RedisKey.TOKEN_KEY;
import static com.abi.qms.platform.infrastructure.util.JwtTokenUtils.checkToken;

/**
 * @author zd
 * @date 2021/1/20 17:57
 */
@Configuration
public class TokenInterceptor implements HandlerInterceptor {

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private UserUtil userUtil;

    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object object) throws Exception {
        // 如果不是映射到方法直接通过
        if (!(object instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) object;
        Method method = handlerMethod.getMethod();
        //检查是否有passtoken注释，有则跳过认证
        if (method.isAnnotationPresent(PassToken.class)) {
            PassToken passToken = method.getAnnotation(PassToken.class);
            if (passToken.required()) {
                return true;
            }
        }
        //检查有没有需要用户权限的注解
        // 从 http 请求头中取出 token
        String token = httpServletRequest.getHeader("Authorization");

        // 执行认证
        if (ObjectUtils.isEmpty(token)||!token.startsWith("Bearer ")) {
            throw new BusinessException(401,"token 信息错误,认证失败");
        }
        token = token.replace("Bearer ","");
        checkToken(token);
        // 获取 token 中的 user id
        String userId;
        try {
            DecodedJWT jwt = JWT.decode(token);
            Map<String, Claim> userMap = jwt.getClaims();
            userId = jwt.getAudience().get(0);
            String redisTokenKey = MessageFormat.format("{0}:{1}",TOKEN_KEY,token);
            Object flag = redisClient.get(redisTokenKey);
            if (ObjectUtils.isEmpty(flag)){
                throw new BusinessException(401,"token已过期,请重新登录");
            }
            redisClient.expire(redisTokenKey,2, TimeUnit.HOURS);
            String userName = userMap.get("userName").asString();
            String roleCode = userMap.get("roleCode").asString();
            UserInfo userInfo = new UserInfo();
            userInfo.setId(Long.valueOf(userId));
            userInfo.setUserName(userName);
            userInfo.setRoleCode(roleCode);
            userUtil.setUser(userInfo);
            TokenUtil.setUser(userInfo);
        } catch (JWTDecodeException j) {
            throw new BusinessException(401,"用户认证失败");
        }
        return true;

    }

    @Override
    public void postHandle(HttpServletRequest httpServletRequest,
                           HttpServletResponse httpServletResponse,
                           Object o, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest,
                                HttpServletResponse httpServletResponse,
                                Object o, Exception e) throws Exception {

        userUtil.removeLocalCache();

    }
}
