package com.abi.qms.platform.infrastructure.task;

import com.abi.qms.platform.dao.entity.QrFormat;
import com.abi.qms.platform.dao.enums.QrFormatUniqueStatusEunm;
import com.abi.qms.platform.dao.enums.QrFormatVerifyUniqueEnum;
import com.abi.qms.platform.dao.mapper.QrFormatMapper;
import com.abi.qms.platform.infrastructure.qr.build.parent.SerialBuildCode;
import com.abi.qms.platform.infrastructure.util.BuildCodeUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.*;

/**
 * 验证重复率线程
 *
 * @author WeiganCai
 * @date 2021-04-21
 */
@Slf4j
public class VerifyUniqueTask implements Runnable {

    /** 测试重复率时生成码的数量 */
    private static final int BUILD_CODE_NUMBER = 500000;

    /** 允许单个码生成重复的次数 */
    private static final int REPEAT_BUILD_CODE_TIMES = 10;

    private QrFormat qrFormat;

    private QrFormatMapper qrFormatMapper;

    public VerifyUniqueTask(QrFormat qrFormat, QrFormatMapper qrFormatMapper) {
        this.qrFormat = qrFormat;
        this.qrFormatMapper = qrFormatMapper;
    }

    @Override
    public void run() {
        log.info("正在对[id:{}, name:{}]码格式进行重复率验证", qrFormat.getId(), qrFormat.getName());

        // 验证开始时间
        long startTime = System.currentTimeMillis();
        // 码格式验证标识
        boolean flag = true;

        try {
            // 验证中
            QrFormat update = new QrFormat().setId(qrFormat.getId()).setUniqueStatus(QrFormatUniqueStatusEunm.VERIFYING.getCode());
            qrFormatMapper.updateById(update);

            // 获取用于生成码的class类
            String[] buildClassArr = qrFormat.getCodeVariableBuildClass().split(",");
            // 用Set存放不重复的码
            HashSet<String> uniqueSet = new HashSet<>();
            // 生成码要用的参数
            Map<String, Object> paramMap = new HashMap<>();
            // 码
            StringBuilder codeSb = new StringBuilder();

            for (int i = 0; i < BUILD_CODE_NUMBER; i++) {
                String code = buildUniqueCode(buildClassArr, uniqueSet, codeSb, paramMap, 0);
                if (code == null) {
                    log.info("第{}个码生成失败", i + 1);
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            log.error("码格式验证error", e);
            flag = false;
        } finally {
            // 修改重复率验证状态
            Integer uniqueStatus = flag ? QrFormatUniqueStatusEunm.VERIFY_SUCCESS.getCode() : QrFormatUniqueStatusEunm.VERIFY_FAIL.getCode();
            QrFormat update = new QrFormat().setId(qrFormat.getId()).setUniqueStatus(uniqueStatus);
            qrFormatMapper.updateById(update);

            log.info("[id:{}, name:{}]码格式重复率验证{}，耗时{}ms", qrFormat.getId(), qrFormat.getName(),
                    flag ? "通过" : "不通过", System.currentTimeMillis() - startTime );
        }

    }

    /**
     * 生成不重复的码
     */
    private String buildUniqueCode(String[] buildClassArr, HashSet<String> uniqueSet, StringBuilder codeSb, Map<String, Object> paramMap, int buildTimes) {
        // 生成码
        buildCodeByClass(buildClassArr, codeSb, paramMap);
        String code = codeSb.toString();

        // add成功，说明码没有重复
        if (uniqueSet.add(code)) {
            return code;
        }

        // 单个码重复次数大于上限
        if (buildTimes >= REPEAT_BUILD_CODE_TIMES) {
            return null;
        }

        // 码重复，递归执行下一次
        return buildUniqueCode(buildClassArr, uniqueSet, codeSb, paramMap, ++buildTimes);
    }

    /**
     * 清空codeSb，调用buildSingleCode方法生成码
     */
    private void buildCodeByClass(String[] buildClassArr, StringBuilder codeSb, Map<String, Object> paramMap) {
        // 使用一个对象，删除内容，不每次都新建一个对象
        codeSb.delete(0, codeSb.length());
        for (String buildClass : buildClassArr) {
            String code = BuildCodeUtil.buildSingleCode(buildClass, paramMap);
            // 如果是流水号类型的变量，需要将返回的流水号存入map中，使流水号连贯
            if (buildClass.startsWith(SerialBuildCode.SERIAL_START_WITH)) {
                paramMap.put(buildClass, code);
            }
            codeSb.append(code);
        }
    }

}
