package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.BaseDepartment;
import com.abi.qms.platform.dao.enums.ValidEnum;
import com.abi.qms.platform.dao.mapper.BaseDepartmentMapper;
import com.abi.qms.platform.dto.req.DisableDeptReq;
import com.abi.qms.platform.dto.req.EnableDeptReq;
import com.abi.qms.platform.dto.req.GetDeptDetailReq;
import com.abi.qms.platform.dto.req.ListDeptReq;
import com.abi.qms.platform.dto.req.SaveDeptReq;
import com.abi.qms.platform.dto.res.ExcelImportRes;
import com.abi.qms.platform.dto.res.GetDeptDetailRes;
import com.abi.qms.platform.dto.res.ListDeptEnableRes;
import com.abi.qms.platform.dto.res.ListDeptRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.DepartmentService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.excel.preperties.DepartmentProperty;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 部门管理
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class DepartmentServiceImpl implements DepartmentService {

    @Autowired
    private BaseDepartmentMapper baseDepartmentMapper;

    /**
     * 部门新增修改
     **/
    @Override
    public void saveDept(SaveDeptReq req) {

        //0-校验入参
        //code不能重复
        QueryWrapper<BaseDepartment> deptQw = new QueryWrapper<>();
        deptQw.eq("dept_code", req.getDeptCode());
        if (req.getId() != null) {
            deptQw.ne("id", req.getId());
        }
        List<BaseDepartment> checkDeptList = baseDepartmentMapper.selectList(deptQw);
        if (CollectionUtil.isNotEmpty(checkDeptList)) {
            throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), "部门ID已存在");
        }

        //1-声明对象
        BaseDepartment dept = null;
        if (req.getId() != null) {
            dept = baseDepartmentMapper.selectById(req.getId());
            AssertUtil.isNull(dept, "部门不存在");
        } else {
            dept = new BaseDepartment();
        }

        //2-放入数据
        dept.setDeptCode(req.getDeptCode());
        dept.setDeptName(req.getDeptName());
        dept.setDeptUrl(req.getDeptUrl());
        dept.setDeptOfficialAccount(req.getDeptOfficialAccount());
        dept.setDescription(req.getDescription());

        //3-新增or修改
        if (req.getId() != null) {
            baseDepartmentMapper.updateById(dept);
        } else {
            baseDepartmentMapper.insert(dept);
        }

    }

    /**
     * 部门查询分页
     **/
    @Override
    public ListDeptRes listDept(ListDeptReq req) {
        //1-拼接条件
        QueryWrapper<BaseDepartment> eventQw = new QueryWrapper<>();
        eventQw.like(StringUtils.isNotBlank(req.getDeptName()), "dept_name", "%" + req.getDeptName() + "%");
        eventQw.eq(req.getValid() != null, "valid", req.getValid());

        //2-分页查询
        IPage iPage = baseDepartmentMapper.selectPage(PageUtil.createPage(req), eventQw);
        List<BaseDepartment> deptList = iPage.getRecords();

        //封装出参
        ListDeptRes res = new ListDeptRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListDeptRes.DeptBean> eventBeanList = PojoConverterUtils.copyList(deptList, ListDeptRes.DeptBean.class);
        res.setDeptBeanList(eventBeanList);

        return res;
    }

    /**
     * 部门查询详情
     **/
    @Override
    public GetDeptDetailRes getDeptDetail(GetDeptDetailReq req) {
        //1-查询活动对象
        BaseDepartment event = baseDepartmentMapper.selectById(req.getId());
        AssertUtil.isNull(event, "部门不存在");

        //转化出参
        GetDeptDetailRes res = PojoConverterUtils.copy(event, GetDeptDetailRes.class);

        return res;
    }

    /**
     * 部门启用
     **/
    @Override
    public void enableDept(EnableDeptReq req) {
        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BaseDepartment dept = baseDepartmentMapper.selectById(id);
            AssertUtil.isNull(dept, "部门不存在");
            if (!ValidEnum.NOT_VALID.is(dept.getValid())) {
                continue;
            }

            //2-修改状态
            dept.setValid(ValidEnum.VALID.getCode());
            baseDepartmentMapper.updateById(dept);
        }
    }

    /**
     * 部门禁用
     **/
    @Override
    public void disableDept(DisableDeptReq req) {
        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BaseDepartment dept = baseDepartmentMapper.selectById(id);
            AssertUtil.isNull(dept, "部门不存在");
            if (!ValidEnum.VALID.is(dept.getValid())) {
                return;
            }

            //2-修改状态
            dept.setValid(ValidEnum.NOT_VALID.getCode());
            baseDepartmentMapper.updateById(dept);
        }
    }

    /**
     * excel批量导入
     **/
    @Override
    public ExcelImportRes deptExcelImport(List<DepartmentProperty> departmentPropertyList) {
        List<ExcelImportRes.FailBean> failList = new ArrayList<>();

        //1-循环导入，有异常则捕获
        for (int i = 0; i < departmentPropertyList.size(); i++) {
            DepartmentProperty departmentProperty = departmentPropertyList.get(i);
            try {
                //1-校验必填与枚举
                String errorMessage = checkDepartmentProperty(departmentProperty);
                if (StringUtils.isNotBlank(errorMessage)) {
                    throw new BusinessException(ErrorCodeEnum.ERROR_PARAM.getCode(), errorMessage);
                }

                //2-调用业务代码新增
                SaveDeptReq req = new SaveDeptReq();
                req.setDeptCode(departmentProperty.getDeptCode());
                req.setDeptName(departmentProperty.getDeptName());
                req.setDeptUrl(departmentProperty.getDeptUrl());
                req.setDeptOfficialAccount(departmentProperty.getDeptOfficialAccount());
                req.setDescription(departmentProperty.getDescription());

                saveDept(req);
            } catch (BusinessException e) {
                failList.add(new ExcelImportRes.FailBean(i + 2, e.getMessage()));
            }
        }

        //2-返回导入结果
        ExcelImportRes res = new ExcelImportRes();
        res.setFailedCount(failList.size());
        res.setSuccessCount(departmentPropertyList.size() - res.getFailedCount());
        res.setFailBeanList(failList);

        return res;

    }

    @Override
    public ListDeptEnableRes listDeptEnable() {
        QueryWrapper<BaseDepartment> param = new QueryWrapper<>();
        param.eq("is_delete", 0);
        param.eq("valid", 1);
        List<BaseDepartment> baseDepartments = baseDepartmentMapper.selectList(param);
        ListDeptEnableRes listDeptEnableRes = new ListDeptEnableRes();
        List<ListDeptEnableRes.DeptBean> deptBeans = PojoConverterUtils.copyList(baseDepartments, ListDeptEnableRes.DeptBean.class);
        listDeptEnableRes.setDeptBeanList(deptBeans);
        return listDeptEnableRes;
    }

    /**
     * 检查批量导入的内容
     *
     * @param departmentProperty
     * @return
     */
    private String checkDepartmentProperty(DepartmentProperty departmentProperty) {
        StringBuffer errorMessage = new StringBuffer();
        //逐个字段校验
        if (StringUtils.isBlank(departmentProperty.getDeptCode())) {
            errorMessage.append("部门id").append("为空,");
        }
        if (StringUtils.isBlank(departmentProperty.getDeptName())) {
            errorMessage.append("部门名称").append("为空,");
        }

        return errorMessage.toString();
    }


}
