package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.entity.UserResource;
import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dao.entity.UserRoleDetail;
import com.abi.qms.platform.dao.enums.ActivateMiniAppRoleResourceEnum;
import com.abi.qms.platform.dao.enums.RoleLevelEnum;
import com.abi.qms.platform.dao.enums.TrueFalseEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.vo.result.ListRoleVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.RoleManagerService;
import com.abi.qms.platform.service.UserResourceService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * <p>
 * 角色管理
 * </p>
 *
 * @author Andy.Tan
 */
@Service
public class RoleManagerServiceImpl implements RoleManagerService {

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserResourceMapper userResourceMapper;

    @Autowired
    private UserMappingRoleResourceMapper userMappingRoleResourceMapper;

    @Autowired
    private UserResourceService userResourceService;

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private UserRoleDetailMapper userRoleDetailMapper;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private RoleManagerService roleManagerService;

    /**
     * 创建角色
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createRole(CreateRoleReq req) {
        //校验角色名称唯一
        QueryWrapper<UserRole> userRoleQw = new QueryWrapper<>();
        userRoleQw.eq("role_name",req.getRoleName());
        userRoleQw.eq("is_delete",0);
        List<UserRole> userRoleList = userRoleMapper.selectList(userRoleQw);
        if(CollectionUtils.isNotEmpty(userRoleList) && userRoleList.size() > 0){
            throw new BusinessException("角色名称" + req.getRoleName() + "已存在。");
        }
        //1-查询上级角色信息
        UserRole parentRole = getRoleByCode(req.getParentRoleCode());
        if(RoleLevelEnum.LEVEL_5.is(parentRole.getRoleLevel())){
            throw new BusinessException("父类是最底层角色，不可继续创建角色");
        }

        //2-插入角色信息
        UserRole userRole = new UserRole();
        userRole.setRoleName(req.getRoleName());
        userRole.setRoleDesc(req.getRoleDesc());
        userRole.setRoleLevel(parentRole.getRoleLevel()+1);
        userRole.setParentRoleCode(req.getParentRoleCode());
        userRoleMapper.insert(userRole);

        //3-放入角色code以及对应level的角色code
        userRole.setRoleCode(generateRoleCode(userRole.getId(),userRole.getRoleLevel()));
        setUserLevelRoleCode(userRole,parentRole);
        userRoleMapper.updateById(userRole);

        //4-放入角色资源mapping
        userResourceService.saveUserResourceMapping(userRole.getRoleCode(),req.getResourceCodeList());

        //5-保存角色detail扩展信息子表（为何设计子表？防止角色过渡臃肿，又必须实现产品提出的特殊需求如小程序主子账号）
        List<Integer> miniList = req.getActivateMiniAppResourceCodeList();
        int isMainAccount = miniList!=null && miniList.contains(ActivateMiniAppRoleResourceEnum.MAIN.getCode())?1:0;
        int isSubAccount = miniList!=null && miniList.contains(ActivateMiniAppRoleResourceEnum.SUB.getCode())?1:0;
        saveRoleDetail(userRole.getRoleCode(),isMainAccount,isSubAccount);

    }

    /**
     * 保存角色明细表
     * @param roleCode
     * @param isMainAccount
     * @param isSubAccount
     */
    private void saveRoleDetail(String roleCode,Integer isMainAccount,Integer isSubAccount){
        //1-查询是否存在detail
        UserRoleDetail roleDetail = roleManagerService.getRoleDetail(roleCode);
        boolean isAdd = roleDetail==null;
        if(isAdd){
            roleDetail = new UserRoleDetail();
        }

        //2-放入值
        roleDetail.setRoleCode(roleCode);
        roleDetail.setIsActivateMiniAppMainAccount(isMainAccount);
        roleDetail.setIsActivateMiniAppSubAccount(isSubAccount);

        //3-修改or新增
        if(isAdd){
            userRoleDetailMapper.insert(roleDetail);
        }else{
            userRoleDetailMapper.updateById(roleDetail);
        }

    }

    /**
     * 获取角色明细
     * @param roleCode
     * @return
     */
    @Override
    public UserRoleDetail getRoleDetail(String roleCode){
        QueryWrapper<UserRoleDetail> roleDetailQw = new QueryWrapper<>();
        roleDetailQw.eq("role_code",roleCode);
        roleDetailQw.eq("is_delete",0);
        UserRoleDetail roleDetail = userRoleDetailMapper.selectOne(roleDetailQw);

        return roleDetail;
    }


    /**
     * 构造角色code
     * 规则：level+n个0+id
     * @param roleId
     * @param roleLevel
     * @return
     */
    private String generateRoleCode(Long roleId,Integer roleLevel){
        return roleLevel+ StringUtils.leftPad(String.valueOf(roleId), 10, "0");
    }

    /**
     * 填充level1-5的角色code
     * @param userRole
     * @param parentUserRole
     */
    private void setUserLevelRoleCode(UserRole userRole,UserRole parentUserRole){
        //1-先循环把父类的级别拷贝过来
        userRole.setL1RoleCode(parentUserRole.getL1RoleCode());
        userRole.setL2RoleCode(parentUserRole.getL2RoleCode());
        userRole.setL3RoleCode(parentUserRole.getL3RoleCode());
        userRole.setL4RoleCode(parentUserRole.getL4RoleCode());
        userRole.setL5RoleCode(parentUserRole.getL3RoleCode());

        //2-然后根据自己的层级，填充对应那一层的code
        if(RoleLevelEnum.LEVEL_1.is(userRole.getRoleLevel())){
            userRole.setL1RoleCode(userRole.getRoleCode());
        }else if(RoleLevelEnum.LEVEL_2.is(userRole.getRoleLevel())){
            userRole.setL2RoleCode(userRole.getRoleCode());
        }else if(RoleLevelEnum.LEVEL_3.is(userRole.getRoleLevel())){
            userRole.setL3RoleCode(userRole.getRoleCode());
        }else if(RoleLevelEnum.LEVEL_4.is(userRole.getRoleLevel())){
            userRole.setL4RoleCode(userRole.getRoleCode());
        }else if(RoleLevelEnum.LEVEL_5.is(userRole.getRoleLevel())){
            userRole.setL5RoleCode(userRole.getRoleCode());
        }
    }

    /**
     * 修改角色
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRole(UpdateRoleReq req) {
        if("admin".equals(req.getRoleCode())){
            throw new BusinessException("超级管理员不可编辑。");
        }
        //1-查询角色信息
        UserRole userRole = getRoleByCode(req.getRoleCode());
        AssertUtil.isNull(userRole,"角色不存在");

        //2-修改角色信息
        userRole.setRoleName(req.getRoleName());
        userRole.setRoleDesc(req.getRoleDesc());
        userRoleMapper.updateById(userRole);

        //3-放入角色资源mapping
        userResourceService.saveUserResourceMapping(userRole.getRoleCode(),req.getResourceCodeList());

        //4-保存角色明细
        List<Integer> miniList = req.getActivateMiniAppResourceCodeList();
        int isMainAccount = miniList!=null && miniList.contains(ActivateMiniAppRoleResourceEnum.MAIN.getCode())?1:0;
        int isSubAccount = miniList!=null && miniList.contains(ActivateMiniAppRoleResourceEnum.SUB.getCode())?1:0;
        saveRoleDetail(userRole.getRoleCode(),isMainAccount,isSubAccount);
    }

    /**
     * 查询单个角色信息
     **/
    @Override
    public GetRoleInfoRes getRoleInfo(GetRoleInfoReq req) {
        //1-查询角色信息
        UserRole userRole = getRoleByCode(req.getRoleCode());
        AssertUtil.isNull(userRole,"角色不存在");

        //2-角色资源列表（仅最底层那一级别的资源）
        List<String> resourceCodeList = userResourceService.listResourceByRoleCode(userRole.getRoleCode());

        //3-小程序的特殊“资源”列表
        List<Integer> miniAppResourceCodeList = new ArrayList<>();
        UserRoleDetail roleDetail = roleManagerService.getRoleDetail(userRole.getRoleCode());
        if(roleDetail!=null){
            if(TrueFalseEnum.TRUE.is(roleDetail.getIsActivateMiniAppMainAccount())){
                miniAppResourceCodeList.add(ActivateMiniAppRoleResourceEnum.MAIN.getCode());
            }
            if(TrueFalseEnum.TRUE.is(roleDetail.getIsActivateMiniAppSubAccount())){
                miniAppResourceCodeList.add(ActivateMiniAppRoleResourceEnum.SUB.getCode());
            }
        }

        //构造出参
        GetRoleInfoRes res = PojoConverterUtils.copy(userRole,GetRoleInfoRes.class);
        res.setResourceCodeList(resourceCodeList);
        res.setActivateMiniAppResourceCodeList(miniAppResourceCodeList);

        //查询启用中的下级角色列表
        List<UserRole> subRoles = userRoleMapper.subordinateRoleEnable(userRole.getRoleCode(), userRole.getRoleLevel());
        if(CollectionUtils.isNotEmpty(subRoles)){
            List<GetRoleInfoRes.RoleInfoBean> roleInfoBeans = PojoConverterUtils.copyList(subRoles, GetRoleInfoRes.RoleInfoBean.class);
            res.setSubRoleInfoList(roleInfoBeans);
        }

        return res;
    }

    /**
     * 查询角色列表
     **/
    @Override
    public ListRoleRes listRole(ListRoleReq req) {
        //获取当前登录的用户
        UserInfo user = userUtil.getUser();
        //获取当前用户的角色
        UserRole role = getRoleByCode(user.getRoleCode());

        //1-查询角色信息
        IPage<ListRoleVO> roleVOPage = userRoleMapper.listRoleFromRootRoleCode(PageUtil.createPage(req),req, role.getRoleCode(), role.getRoleLevel());
        List<ListRoleVO> roleVOList = roleVOPage.getRecords();

        //2-循环补充该角色有多少用户
        List<ListRoleRes.RoleBean> roleBeanList = PojoConverterUtils.copyList(roleVOList, ListRoleRes.RoleBean.class);
        for(ListRoleRes.RoleBean roleBean:roleBeanList){
            Long userCount = userRoleMapper.countList(roleBean.getRoleCode());
            roleBean.setUserCount(userCount);
        }

        //构造出参
        ListRoleRes res = new ListRoleRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, roleVOPage);
        res.setRoleBeanList(roleBeanList);

        return res;
    }

    /**
     * 角色code查询角色信息
     * @param roleCode
     * @return
     */
    @Override
    public UserRole getRoleByCode(String roleCode) {
        QueryWrapper<UserRole> userRoleQw = new QueryWrapper<>();
        userRoleQw.eq("role_code",roleCode);
        userRoleQw.eq("is_delete",0);
        List<UserRole> userRoleList = userRoleMapper.selectList(userRoleQw);
        if(CollectionUtils.isEmpty(userRoleList)){
            return null;
        }

        return userRoleList.get(0);
    }

    /**
     * 查询当前登陆用户角色的resource树
     **/
    @Override
    public ListResourceTreeByRoleCodeRes listResourceTreeByRoleCode(ListResourceTreeByRoleCodeReq req) {
        //获取当前登录的用户
        UserInfo user = userUtil.getUser();
        //获取当前用户的角色
        UserRole role = getRoleByCode(user.getRoleCode());

        //1-查询角色树
        List<UserResource> userResourceList = userResourceMapper.listResourceTreeByRoleCode(role.getRoleCode(),null);

        //构造出参
        ListResourceTreeByRoleCodeRes res = new ListResourceTreeByRoleCodeRes();
        List<ListResourceTreeByRoleCodeRes.ResourceBean> resourceBeanList = new ArrayList<>();

        //2-转树形
        if(req.getIsTree()!=null && req.getIsTree()==1){
            //放一个map，用于快捷查找到list  key为父resourceCode
            Map<String,List<ListResourceTreeByRoleCodeRes.ResourceBean>> resourceListMap = new HashMap<>();
            //如果父类code为null，那么父list就是顶层list
            resourceListMap.put(null,resourceBeanList);
            //开始循环放入，因为查询语句排过序，所以查到的顺序，一定是先父后子
            for(UserResource resource:userResourceList){
                //先获取父类列表
                List<ListResourceTreeByRoleCodeRes.ResourceBean> parentList = resourceListMap.get(resource.getParentResourceCode());
                //然后放入自己
                ListResourceTreeByRoleCodeRes.ResourceBean resourceBean = PojoConverterUtils.copy(resource, ListResourceTreeByRoleCodeRes.ResourceBean.class);
                parentList.add(resourceBean);
                //如果resourceCode是3位or6位，则需要创建子list
                if(resourceBean.getResourceCode().length()<=6){
                    resourceBean.setResourceBeanList(new ArrayList<>());
                    resourceListMap.put(resourceBean.getResourceCode(),resourceBean.getResourceBeanList());
                }
            }
        }else{
            resourceBeanList = PojoConverterUtils.copyList(userResourceList, ListResourceTreeByRoleCodeRes.ResourceBean.class);
        }

        //放入出参
        res.setResourceBeanList(resourceBeanList);

        return res;
    }

    @Override
    public ListResourceTreeBySelectRoleCodeRes listResourceTreeBySelectRoleCode(ListResourceTreeBySelectRoleCodeReq listResourceTreeBySelectRoleCodeReq) throws Exception {
        List<UserResource> userResourceList;
        //1-查询角色树  如果roleCode是admin，查出全部资源
        if("admin".equals(listResourceTreeBySelectRoleCodeReq.getRoleCode())){
            userResourceList = userResourceMapper.listAll(null);
        }else{
            userResourceList = userResourceMapper.listResourceTreeByRoleCode(listResourceTreeBySelectRoleCodeReq.getRoleCode(),null);
        }

        //构造出参
        ListResourceTreeBySelectRoleCodeRes res = new ListResourceTreeBySelectRoleCodeRes();
        List<ListResourceTreeBySelectRoleCodeRes.ResourceBean> resourceBeanList = new ArrayList<>();

        //2-转树形
        if (listResourceTreeBySelectRoleCodeReq.getIsTree() != null && listResourceTreeBySelectRoleCodeReq.getIsTree() == 1) {
            //放一个map，用于快捷查找到list  key为父resourceCode
            Map<String, List<ListResourceTreeBySelectRoleCodeRes.ResourceBean>> resourceListMap = new HashMap<>();
            //如果父类code为null，那么父list就是顶层list
            resourceListMap.put(null, resourceBeanList);
            //开始循环放入，因为查询语句排过序，所以查到的顺序，一定是先父后子
            for (UserResource resource : userResourceList) {
                //先获取父类列表
                List<ListResourceTreeBySelectRoleCodeRes.ResourceBean> parentList = resourceListMap.get(resource.getParentResourceCode());
                //然后放入自己
                ListResourceTreeBySelectRoleCodeRes.ResourceBean resourceBean = PojoConverterUtils.copy(resource, ListResourceTreeBySelectRoleCodeRes.ResourceBean.class);
                parentList.add(resourceBean);
                //如果resourceCode是3位or6位，则需要创建子list
                if (resourceBean.getResourceCode().length() <= 6) {
                    resourceBean.setResourceBeanList(new ArrayList<>());
                    resourceListMap.put(resourceBean.getResourceCode(), resourceBean.getResourceBeanList());
                }
            }
        }else {
            resourceBeanList = PojoConverterUtils.copyList(userResourceList, ListResourceTreeBySelectRoleCodeRes.ResourceBean.class);
        }

        //放入出参
        res.setResourceBeanList(resourceBeanList);

        return res;
    }

    /**
     * 获取登录用户需要的页面以及按钮权限
     **/
    @Override
    public ListLoginUserResourceRes listLoginUserResource() {
        //获取当前登录的用户
        UserInfo user = userUtil.getUser();
        //获取当前用户的角色
        UserRole role = getRoleByCode(user.getRoleCode());

        //1-查询菜单
        List<UserResource> userResourceList = userResourceMapper.listLoginMenu(role.getRoleCode());

        //2-查询按钮
        List<UserResource> btnResourceList = userResourceMapper.listResourceTreeByRoleCode(role.getRoleCode(),2);

        //构造出参
        ListLoginUserResourceRes res = new ListLoginUserResourceRes();
        res.setMenuBeanList(PojoConverterUtils.copyList(userResourceList,ListLoginUserResourceRes.MenuBean.class));
        res.setBtnBeanList(PojoConverterUtils.copyList(btnResourceList,ListLoginUserResourceRes.BtnBean.class));

        return res;
    }

    /**
     * 启用角色
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableRole(EnableRoleReq req) {
        List<String> roleCodeList = req.getRoleCodeList();

        //循环启用
        for(String roleCode:roleCodeList){
            //1-查询角色
            UserRole role = getRoleByCode(roleCode);
            AssertUtil.isNull(role,"角色不存在");
            if(role.getIsEnable()==1){
                continue;
            }
            //查询禁用的上级角色
            List<UserRole> userRoles = userRoleMapper.superiorRoleDisable(roleCode);
            if(Objects.nonNull(userRoles) && userRoles.size() > 0){
                throw new BusinessException("角色" + role.getRoleName() + "的上级角色为停用状态，无法启用。");
            }
            //2-修改状态
            role.setIsEnable(1);
            userRoleMapper.updateById(role);
        }

    }

    /**
     * 禁用角色
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void disableRole(DisableRoleReq req) {
        List<String> roleCodeList = req.getRoleCodeList();

        //循环禁用
        for(String roleCode:roleCodeList){
            if("admin".equals(roleCode)){
                throw new BusinessException("超级管理员不可停用。");
            }
            //1-查询角色
            UserRole role = getRoleByCode(roleCode);
            AssertUtil.isNull(role,"角色不存在");
            if(role.getIsEnable()==0){
                continue;
            }
            //查询该角色下启用中的用户
            List<UserInfo> userInfos = userInfoMapper.selectUserInfoByRoleCodeEnable(roleCode);
            if(Objects.nonNull(userInfos) && userInfos.size() > 0){
                throw new BusinessException("角色" + role.getRoleName() + "已与" + userInfos.size() + "名用户绑定，请解绑后再停用。");
            }

            //查询启用中的下级角色
            List<UserRole> userRoles = userRoleMapper.subordinateRoleEnable(role.getRoleCode(), role.getRoleLevel());
            if(Objects.nonNull(userRoles) && userRoles.size() > 0){
                throw new BusinessException("角色" + role.getRoleName() + "存在未停用子角色" + userRoles.get(0).getRoleName() + "，请停用子角色后再试。");
            }

            //2-修改状态
            role.setIsEnable(0);
            userRoleMapper.updateById(role);
        }

    }
}
