package com.abi.qms.platform.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abi.qms.platform.dao.entity.*;
import com.abi.qms.platform.dao.enums.TrueFalseEnum;
import com.abi.qms.platform.dao.mapper.*;
import com.abi.qms.platform.dao.vo.result.ListSubAccountVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.*;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.RoleManagerService;
import com.abi.qms.platform.service.UserManagerService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@Service
public class UserManagerServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserManagerService {

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserMappingInfoRoleMapper userMappingInfoRoleMapper;

    @Autowired
    private BaseFactoryMapper baseFactoryMapper;

    @Autowired
    private RoleManagerService roleManagerService;

    @Autowired
    private UserUtil userUtil;

    @Autowired
    private UserRoleDetailMapper userRoleDetailMapper;

    /**
     * 创建用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createUser(CreateUserReq req) {
        //0-校验入参
        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", req.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            throw new BusinessException("该角色不存在。");
        }
        if (lambdaQuery().eq(UserInfo::getEmployeeNo, req.getEmployeeNo().trim()).count() > 0) {
            throw new BusinessException("工号" + req.getEmployeeNo() + "已存在，请勿重复创建。");
        }
        if (lambdaQuery().eq(UserInfo::getUserName, req.getUserName().trim()).count() > 0) {
            throw new BusinessException("用户" + req.getUserName() + "已存在，请勿重复创建。");
        }
        if(!TrueFalseEnum.TRUE.is(req.getIsAbi()) && req.getFactoryId()==null){
            throw new BusinessException("非ABI用户必须选择供应商");
        }
        //查询供应商
        Integer factoryType = null;
        if(req.getFactoryId()!=null){
            BaseFactory factory = baseFactoryMapper.selectById(req.getFactoryId());
            AssertUtil.isNull(factory,"供应商不存在");
            factoryType = factory.getFactoryType();
        }

        //操作人
        UserInfo loginUser = userUtil.getWholeUser();

        //1-插入user
        UserInfo userInfo = new UserInfo();
        userInfo.setUserName(req.getUserName());
        userInfo.setEmployeeNo(req.getEmployeeNo());
        userInfo.setEmail(req.getEmail());
        userInfo.setPassword(req.getPassword());
        userInfo.setIsAbi(req.getIsAbi());
        userInfo.setFactoryId(req.getFactoryId());
        //添加用户供应商类型的标志
        userInfo.setFactoryType(factoryType);
        userInfo.setRoleCode(req.getRoleCode());
        userInfo.setPhone(req.getPhone());
        userInfoMapper.insert(userInfo);
        //放入link flag链接，空间换时间
        userInfo.setLinkFlag(loginUser.getLinkFlag() + userInfo.getId() + ",");
        userInfoMapper.updateById(userInfo);

        //2-添加用户和角色的映射关系
        UserMappingInfoRole userMappingInfoRole = new UserMappingInfoRole();
        userMappingInfoRole.setUserId(userInfo.getId());
        userMappingInfoRole.setRoleCode(userRole.getRoleCode());
        userMappingInfoRole.setRoleName(userRole.getRoleName());
        userMappingInfoRoleMapper.insert(userMappingInfoRole);
    }

    /**
     * 修改用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUser(UpdateUserReq req) {
        //0-校验入参
        UserInfo userInfo = userInfoMapper.selectById(req.getId());
        if(Objects.isNull(userInfo)){
            throw new BusinessException("该用户不存在。");
        }
        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", req.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            throw new BusinessException("选择的角色不存在。");
        }
        String oriRoleCode = userInfo.getRoleCode();

        //1-修改主表
        userInfo.setEmail(req.getEmail());
        userInfo.setPhone(req.getPhone());
        //校验供应商
        if(Objects.nonNull(req.getFactoryId())) {
            if(!userInfo.getFactoryId().equals(req.getFactoryId())) {
                BaseFactory factory = baseFactoryMapper.selectById(req.getFactoryId());
                AssertUtil.isNull(factory,"供应商不存在");
                userInfo.setFactoryId(req.getFactoryId());
                userInfo.setFactoryType(factory.getFactoryType());
            }
        }
        userInfo.setRoleCode(req.getRoleCode());
        userInfoMapper.updateById(userInfo);

        //2-修改用户和角色的映射关系
        //如果修改了角色，才修改映射
        if(!oriRoleCode.equals(req.getRoleCode())) {
            //先查询原有映射
            QueryWrapper<UserMappingInfoRole> mapQw = new QueryWrapper<>();
            mapQw.eq("user_id", userInfo.getId());
            mapQw.eq("is_delete", 0);
            UserMappingInfoRole userMappingInfoRole = userMappingInfoRoleMapper.selectOne(mapQw);
            //正常逻辑是修改映射
            if (Objects.nonNull(userMappingInfoRole)) {
                //修改印射
                userMappingInfoRole.setRoleCode(userRole.getRoleCode());
                userMappingInfoRole.setRoleName(userRole.getRoleName());
                userMappingInfoRoleMapper.updateById(userMappingInfoRole);
            }else{
                //如果是手动造的，或者被改动的不正确的数据，没有印射，则新增
                userMappingInfoRole = new UserMappingInfoRole();
                userMappingInfoRole.setUserId(userInfo.getId());
                userMappingInfoRole.setRoleCode(userRole.getRoleCode());
                userMappingInfoRole.setRoleName(userRole.getRoleName());
                userMappingInfoRoleMapper.insert(userMappingInfoRole);
            }
        }
    }

    /**
     * 查询角色列表
     **/
    @Override
    public ListRoleTreeRes listRoleTree() {
        //获取当前登录的用户
        UserInfo user = userUtil.getUser();
        //获取当前用户的角色
        UserRole role = roleManagerService.getRoleByCode(user.getRoleCode());

        //1-查询当前角色以及其下的所有角色
        List<UserRole> userRoleList = userRoleMapper.listRoleTreeEnable(role);

        //构造出参
        List<ListRoleTreeRes.RoleBean> roleBeanList = PojoConverterUtils.copyList(userRoleList, ListRoleTreeRes.RoleBean.class);
        ListRoleTreeRes res = new ListRoleTreeRes();
        res.setRoleBeanList(roleBeanList);

        return res;
    }


    /**
     * 查询单个用户信息
     **/
    @Override
    public GetUserInfoRes getUserInfo(Long userInfoId) {
        //查询用户
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);

        //构造出参
        GetUserInfoRes getUserInfoRes = PojoConverterUtils.copy(userInfo,GetUserInfoRes.class);

        return getUserInfoRes;
    }

    /**
     * 查询用户列表
     **/
    @Override
    public ListUserRes listUser(ListUserReq listUserReq) {
        //当前登录人
        UserInfo user = userUtil.getUser();
        //放入数据链标记（效果是仅能看到自己这条链路下的用户）
        listUserReq.setLinkFlag("," + user.getId() + ",");

        //1-分页查询
        IPage iPage = userInfoMapper.selectUserInfoPage(PageUtil.createPage(listUserReq), listUserReq);
        List<UserInfo> records = iPage.getRecords();

        //封装出参
        ListUserRes res = new ListUserRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListUserRes.UserInfoBean> eventBeanList = PojoConverterUtils.copyList(records, ListUserRes.UserInfoBean.class);
        res.setUserInfoBeanList(eventBeanList);

        return res;
    }

    /**
     * 启用/停用用户
     * @param userInfoId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableUser(Long userInfoId) {
        //查询用户信息
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);
        AssertUtil.isNull(userInfo,"用户不存在");

        //状态反一反。。。
        userInfo.setIsEnable(1 - userInfo.getIsEnable());
        userInfoMapper.updateById(userInfo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchEnableUser(BatchEnableUserReq req) throws Exception {
        userInfoMapper.batchEnableUser(req);
    }

    @Override
    public GetUserInfoWxRes getUserInfoWx() {
        //查询用户
        UserInfo userInfo = userInfoMapper.selectById(userUtil.getUser().getId());

        //构造出参
        GetUserInfoWxRes getUserInfoWxRes = PojoConverterUtils.copy(userInfo,GetUserInfoWxRes.class);
        getUserInfoWxRes.setIsMainAccount(userUtil.getWholeUser().getIsMainAccount());

        return getUserInfoWxRes;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePhoneEmailWx(UpdatePhoneEmailWxReq req) {
        //0-校验入参
        UserInfo userInfo = userInfoMapper.selectById(userUtil.getUser().getId());
        AssertUtil.isNull(userInfo, "该用户不存在。");
        userInfo.setEmail(req.getEmail());
        userInfo.setPhone(req.getPhone());
        userInfo.setUpdateBy(userUtil.getUser().getId());
        userInfoMapper.updateById(userInfo);
    }

    @Override
    public ListSubAccountWxRes listSubAccountWx() {
        ListSubAccountWxRes listSubAccountWxRes = new ListSubAccountWxRes();
        List<ListSubAccountVO> listSubAccountVOList = userInfoMapper.selectSubAccountList(userUtil.getWholeUser().getFactoryId());
        if (CollectionUtil.isNotEmpty(listSubAccountVOList)) {
            List<ListSubAccountWxRes.SubAccount> subAccountList = new ArrayList<>();
            //循环构造出参
            listSubAccountVOList.forEach(listSubAccountVO -> {
                ListSubAccountWxRes.SubAccount subAccount = PojoConverterUtils.copy(listSubAccountVO, ListSubAccountWxRes.SubAccount.class);
                List<ListSubAccountWxRes.ProductionLineName> productionLineNames = PojoConverterUtils.copyList(listSubAccountVO.getProductionLineNameList(), ListSubAccountWxRes.ProductionLineName.class);
                subAccount.setProductionLineNameList(productionLineNames);
                subAccountList.add(subAccount);
            });
            listSubAccountWxRes.setSubAccountList(subAccountList);
        }
        return listSubAccountWxRes;
    }

    @Override
    public GetSubAccountWxRes subAccountDetailWx(Long subUserInfoId) {
        ListSubAccountVO listSubAccountVO = userInfoMapper.selectSubAccountDetail(subUserInfoId);
        GetSubAccountWxRes getSubAccountWxRes = new GetSubAccountWxRes();
        if(Objects.nonNull(listSubAccountVO)){
            getSubAccountWxRes = PojoConverterUtils.copy(listSubAccountVO, GetSubAccountWxRes.class);
            List<GetSubAccountWxRes.ProductionLineName> productionLineNames = PojoConverterUtils.copyList(listSubAccountVO.getProductionLineNameList(), GetSubAccountWxRes.ProductionLineName.class);
            getSubAccountWxRes.setProductionLineNameList(productionLineNames);
        }
        return getSubAccountWxRes;
    }


}
